@@ -1,8 +1,9 @@
 import urllib
 from codecs import StreamWriter
-from typing import Any, Dict, Optional, TextIO, Union, cast
+from typing import Any, Dict, Iterator, Optional, TextIO, Union, cast
 
 from rdflib import Graph
+from rdflib.query import ResultRow
 from ruamel.yaml.comments import CommentedMap
 from schema_salad.jsonld_context import makerdf
 from schema_salad.utils import ContextType
@@ -26,7 +27,7 @@ def printrdf(wflow: Process, ctx: ContextType, style: str) -> str:
     rdf = gather(wflow, ctx).serialize(format=style, encoding="utf-8")
     if not rdf:
         return ""
-    return cast(str, rdf.decode("utf-8"))
+    return rdf.decode("utf-8")
 
 
 def lastpart(uri: Any) -> str:
@@ -37,13 +38,16 @@ def lastpart(uri: Any) -> str:
 
 
 def dot_with_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> None:
-    qres = g.query(
-        """SELECT ?step ?run ?runtype
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?step ?run ?runtype
            WHERE {
               ?step cwl:run ?run .
               ?run rdf:type ?runtype .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for step, run, _ in qres:
         stdout.write(
@@ -51,14 +55,17 @@ def dot_with_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> None:
             % (lastpart(step), f"{lastpart(step)} ({lastpart(run)})")
         )
 
-    qres = g.query(
-        """SELECT ?step ?inp ?source
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?step ?inp ?source
            WHERE {
               ?wf Workflow:steps ?step .
               ?step cwl:inputs ?inp .
               ?inp cwl:source ?source .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for step, inp, source in qres:
         stdout.write('"%s" [shape=box]\n' % (lastpart(inp)))
@@ -69,13 +76,16 @@ def dot_with_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> None:
             '"{}" -> "{}" [label="{}"]\n'.format(lastpart(inp), lastpart(step), "")
         )
 
-    qres = g.query(
-        """SELECT ?step ?out
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?step ?out
            WHERE {
               ?wf Workflow:steps ?step .
               ?step cwl:outputs ?out .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for step, out in qres:
         stdout.write('"%s" [shape=box]\n' % (lastpart(out)))
@@ -83,13 +93,16 @@ def dot_with_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> None:
             '"{}" -> "{}" [label="{}"]\n'.format(lastpart(step), lastpart(out), "")
         )
 
-    qres = g.query(
-        """SELECT ?out ?source
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?out ?source
            WHERE {
               ?wf cwl:outputs ?out .
               ?out cwl:source ?source .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for out, source in qres:
         stdout.write('"%s" [shape=octagon]\n' % (lastpart(out)))
@@ -97,13 +110,16 @@ def dot_with_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> None:
             '"{}" -> "{}" [label="{}"]\n'.format(lastpart(source), lastpart(out), "")
         )
 
-    qres = g.query(
-        """SELECT ?inp
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?inp
            WHERE {
               ?wf rdf:type cwl:Workflow .
               ?wf cwl:inputs ?inp .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for (inp,) in qres:
         stdout.write('"%s" [shape=octagon]\n' % (lastpart(inp)))
@@ -116,27 +132,33 @@ def dot_without_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> Non
     stdout.write("compound=true\n")
 
     subworkflows = set()
-    qres = g.query(
-        """SELECT ?run
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?run
            WHERE {
               ?wf rdf:type cwl:Workflow .
               ?wf Workflow:steps ?step .
               ?step cwl:run ?run .
               ?run rdf:type cwl:Workflow .
            } ORDER BY ?wf"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
     for (run,) in qres:
         subworkflows.add(run)
 
-    qres = g.query(
-        """SELECT ?wf ?step ?run ?runtype
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT ?wf ?step ?run ?runtype
            WHERE {
               ?wf rdf:type cwl:Workflow .
               ?wf Workflow:steps ?step .
               ?step cwl:run ?run .
               ?run rdf:type ?runtype .
            } ORDER BY ?wf"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     currentwf = None  # type: Optional[str]
     for wf, step, _run, runtype in qres:
@@ -164,8 +186,10 @@ def dot_without_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> Non
     if currentwf is not None:
         stdout.write("}\n")
 
-    qres = g.query(
-        """SELECT DISTINCT ?src ?sink ?srcrun ?sinkrun
+    qres = cast(
+        Iterator[ResultRow],
+        g.query(
+            """SELECT DISTINCT ?src ?sink ?srcrun ?sinkrun
            WHERE {
               ?wf1 Workflow:steps ?src .
               ?wf2 Workflow:steps ?sink .
@@ -175,7 +199,8 @@ def dot_without_parameters(g: Graph, stdout: Union[TextIO, StreamWriter]) -> Non
               ?src cwl:run ?srcrun .
               ?sink cwl:run ?sinkrun .
            }"""
-    )
+        ),
+    )  # ResultRow because the query is of type SELECT
 
     for src, sink, srcrun, sinkrun in qres:
         attr = ""
@@ -194,4 +219,4 @@ def printdot(
     stdout: Union[TextIO, StreamWriter],
 ) -> None:
     cwl_viewer = CWLViewer(printrdf(wf, ctx, "n3"))  # type: CWLViewer
-    stdout.write(cwl_viewer.dot())
+    stdout.write(cwl_viewer.dot().replace(f"{wf.metadata['id']}#", ""))
