@@ -661,9 +661,11 @@ def setup_schema(
             ext10 = res.read().decode("utf-8")
         with pkg_resources.resource_stream(__name__, "extensions-v1.1.yml") as res:
             ext11 = res.read().decode("utf-8")
+        with pkg_resources.resource_stream(__name__, "extensions-v1.2.yml") as res:
+            ext12 = res.read().decode("utf-8")
         use_custom_schema("v1.0", "http://commonwl.org/cwltool", ext10)
         use_custom_schema("v1.1", "http://commonwl.org/cwltool", ext11)
-        use_custom_schema("v1.2", "http://commonwl.org/cwltool", ext11)
+        use_custom_schema("v1.2", "http://commonwl.org/cwltool", ext12)
         use_custom_schema("v1.2.0-dev1", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2.0-dev2", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2.0-dev3", "http://commonwl.org/cwltool", ext11)
@@ -1011,7 +1013,8 @@ def main(
                 argsl = sys.argv[1:]
             addl = []  # type: List[str]
             if "CWLTOOL_OPTIONS" in os.environ:
-                addl = os.environ["CWLTOOL_OPTIONS"].split(" ")
+                c_opts = os.environ["CWLTOOL_OPTIONS"].split(" ")
+                addl = [x for x in c_opts if x != ""]
             parser = arg_parser()
             argcomplete.autocomplete(parser)
             args = parser.parse_args(addl + argsl)
