@@ -9,15 +9,16 @@ import sys
 from io import StringIO
 from pathlib import Path
 from typing import Any, Dict, List, Union, cast
-from urllib.parse import urlparse
 
+import cwl_utils.expression as expr
 import pydot
 import pytest
+from cwl_utils.errors import JavascriptException
+from cwl_utils.sandboxjs import param_re
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.exceptions import ValidationException
 
 import cwltool.checker
-import cwltool.expression as expr
 import cwltool.factory
 import cwltool.pathmapper
 import cwltool.process
@@ -27,7 +28,6 @@ from cwltool.context import RuntimeContext
 from cwltool.errors import WorkflowException
 from cwltool.main import main
 from cwltool.process import CWL_IANA
-from cwltool.sandboxjs import JavascriptException
 from cwltool.utils import CWLObjectType, dedup
 
 from .util import get_data, get_main_output, needs_docker, working_directory
@@ -64,7 +64,7 @@ expression_match = [
 
 @pytest.mark.parametrize("expression,expected", expression_match)
 def test_expression_match(expression: str, expected: bool) -> None:
-    match = expr.param_re.match(expression)
+    match = param_re.match(expression)
     assert (match is not None) == expected
 
 
@@ -910,6 +910,7 @@ def test_static_checker() -> None:
         factory.make(get_data("tests/checker_wf/broken-wf3.cwl"))
 
 
+@needs_docker
 def test_circular_dependency_checker() -> None:
     # check that the circular dependency checker raises exception when there is
     # circular dependency in the workflow.
@@ -1077,19 +1078,20 @@ test_factors = [(""), ("--parallel"), ("--debug"), ("--parallel --debug")]
 
 
 @pytest.mark.parametrize("factor", test_factors)
-def test_js_console_cmd_line_tool(factor: str) -> None:
+def test_js_console_cmd_line_tool(
+    factor: str, caplog: pytest.LogCaptureFixture
+) -> None:
     for test_file in ("js_output.cwl", "js_output_workflow.cwl"):
         commands = factor.split()
         commands.extend(
             ["--js-console", "--no-container", get_data("tests/wf/" + test_file)]
         )
-        error_code, _, stderr = get_main_output(commands)
+        error_code, _, _ = get_main_output(commands)
+        logging_output = "\n".join([record.message for record in caplog.records])
+        assert "[log] Log message" in logging_output
+        assert "[err] Error message" in logging_output
 
-        stderr = re.sub(r"\s\s+", " ", stderr)
-        assert "[log] Log message" in stderr
-        assert "[err] Error message" in stderr
-
-        assert error_code == 0, stderr
+        assert error_code == 0, logging_output
 
 
 @pytest.mark.parametrize("factor", test_factors)
@@ -1118,8 +1120,9 @@ def test_cid_file_dir(tmp_path: Path, factor: str) -> None:
         stderr = re.sub(r"\s\s+", " ", stderr)
         assert "completed success" in stderr
         assert error_code == 0
-        cidfiles_count = sum(1 for _ in tmp_path.glob("**/*"))
-        assert cidfiles_count == 2
+        cidfiles = list(tmp_path.glob("**/*.cid"))
+        cidfiles_count = len(cidfiles)
+        assert cidfiles_count == 2, f"Should be 2 cidfiles, but got {cidfiles}"
 
 
 @needs_docker
@@ -1448,6 +1451,7 @@ def test_no_compute_chcksum(tmp_path: Path, factor: str) -> None:
     assert "checksum" not in stdout
 
 
+@needs_docker
 @pytest.mark.parametrize("factor", test_factors)
 def test_bad_userspace_runtime(factor: str) -> None:
     test_file = "tests/wf/wc-tool.cwl"
@@ -1784,3 +1788,22 @@ def tests_outputsource_valid_identifier_invalid_source() -> None:
     stderr = re.sub(r"\s\s+", " ", stderr)
     assert "tests/checker_wf/broken-wf4.cwl:12:5: outputSource not found" in stderr
     assert "tests/checker_wf/broken-wf4.cwl#echo_w" in stderr
+
+
+def test_mismatched_optional_arrays() -> None:
+    """Ignore 'null' when comparing array types."""
+    factory = cwltool.factory.Factory()
+
+    with pytest.raises(ValidationException):
+        factory.make(get_data("tests/checker_wf/optional_array_mismatch.cwl"))
+
+
+def test_validate_optional_src_with_mandatory_sink() -> None:
+    """Confirm expected warning with an optional File connected to a mandatory File."""
+    exit_code, stdout, stderr = get_main_output(
+        ["--validate", get_data("tests/wf/optional_src_mandatory_sink.cwl")]
+    )
+    assert exit_code == 0
+    stderr = re.sub(r"\s\s+", " ", stderr)
+    assert 'Source \'opt_file\' of type ["null", "File"] may be incompatible' in stderr
+    assert "with sink 'r' of type \"File\"" in stderr
